<?php
namespace tests\Zeedhi\MyMenu\Controller;

use Zeedhi\MyMenu\Service\MyMenu as MyMenuService;
use Zeedhi\MyMenu\Controller\MyMenu as MyMenuController;
use Zeedhi\MyMenu\Controller\Exception as MyMenuException;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

class MyMenuTest extends \PHPUnit_Framework_TestCase {

    protected $helloWorldController;
    protected $helloWorldService;

    public function setUp() {
        $this->helloWorldService = $this->getMockBuilder(MyMenuService::class)
                                        ->setMethods(array('greetings'))
                                        ->disabLeOriginalConstructor()
                                        ->getMock();

        $this->helloWorldController = new MyMenuController($this->helloWorldService);
    }

    public function testGreetings() {
        $name = 'name';
        $row = array(
            'name' => $name
        );
        $request = new Request\Row($row, 'POST', '/route', 'USER_ID');
        $response = new Response();

        $this->helloWorldService->expects($this->once())
                                ->method('greetings')
                                ->with($name)
                                ->will($this->returnValue('Hey, '.$name));

        $this->helloWorldController->greetings($request, $response);

        $dataSets = $response->getDataSets();
        $this->assertCount(1, $dataSets);

        $dataSet = $dataSets[0];
        $this->assertEquals('message', $dataSet->getDataSourceName());

        $rows = $dataSet->getRows();
        $this->assertCount(1, $rows);

        $row = $rows[0];
        $this->assertCount(1, $row);
        $this->assertArrayHasKey('text', $row);
        $this->assertEquals('Hey, '.$name, $row['text']);
    }

    public function testGreetingsError() {
        $name = 'name';
        $row = array(
            'not_name' => $name
        );
        $request = new Request\Row($row, 'POST', '/route', 'USER_ID');
        $response = new Response();

        $this->helloWorldService->expects($this->never())
                                ->method('greetings');

        $this->helloWorldController->greetings($request, $response);

        $dataSets = $response->getDataSets();
        $this->assertCount(0, $dataSets);

        $error = $response->getError();
        $this->assertNotNull($error);

        $this->assertEquals('Missing name property', $error->getMessage());
        $this->assertEquals(MyMenuException::MISSING_NAME, $error->getErrorCode());
    }

}